@@ -33,37 +33,6 @@
 # POSSIBILITY OF SUCH DAMAGE.
 # ----------------------------------------------------------------------------
 
-# Copyright (c) 2006-2007 Alex Holkner
-# All rights reserved.
-# 
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions 
-# are met:
-#
-#  * Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright 
-#    notice, this list of conditions and the following disclaimer in
-#    the documentation and/or other materials provided with the
-#    distribution.
-#  * Neither the name of the pyglet nor the names of its
-#    contributors may be used to endorse or promote products
-#    derived from this software without specific prior written
-#    permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
-# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
-# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
-# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
-
 '''A sprite-based game loosely based on the classic "Asteroids".
 
 Shoot the asteroids, get high score.
@@ -78,12 +47,9 @@ import os
 import random
 import sys
 
+import pyglet
 from pyglet.gl import *
-from pyglet import clock
-from pyglet import font
-from pyglet import image
-from pyglet import media
-from pyglet import window
+from pyglet import resource
 from pyglet.window import key
 
 PLAYER_SPIN_SPEED = 360.
@@ -115,6 +81,8 @@ GET_READY_DELAY = 1.
 BEGIN_PLAY_DELAY = 2.
 LIFE_LOST_DELAY = 2.
 
+FONT_NAME = ('Verdana', 'Helvetica', 'Arial')
+
 INSTRUCTIONS = \
 '''Your ship is lost in a peculiar unchartered area of space-time infested with asteroids!  You have no chance for survival except to rack up the highest score possible.
 
@@ -124,33 +92,9 @@ Space: Shoot
 
 Be careful, there's not much friction in space.'''
 
-class Animation(object):
-    def __init__(self, x, y, dx, dy, images, period):
-        self.x = x
-        self.y = y
-        self.dx = dx
-        self.dy = dy
-        self.images = images
-        self.period = period
-        self.time = 0
-        self.frame = 0
-        self.finished = False
-
-    def update(self, dt):
-        self.x += self.dx * dt
-        self.y += self.dy * dt
-
-        self.time += dt
-        while self.time >= self.period:
-            self.time -= self.period
-            self.frame += 1
-            if self.frame == len(self.images):
-                self.frame = 0 
-                self.finished = True
-
-    def draw(self):
-        img = self.images[self.frame]
-        img.blit(self.x - img.width / 2, self.y - img.height / 2, 0)
+def center_anchor(img):
+    img.anchor_x = img.width // 2
+    img.anchor_y = img.height // 2
 
 # --------------------------------------------------------------------------
 # Game objects
@@ -166,26 +110,23 @@ def wrap(value, width):
 def to_radians(degrees):
     return math.pi * degrees / 180.0
 
-class WrappingObject(object):
+class WrappingSprite(pyglet.sprite.Sprite):
     dx = 0
     dy = 0
-    heading = 0
-    heading_speed = 0
+    rotation_speed = 0
 
-    def __init__(self, x, y, img):
-        self.x = x
-        self.y = y
-        self.img = img
-        self.collision_radius = self.img.width / COLLISION_RESOLUTION / 2 
+    def __init__(self, img, x, y, batch=None):
+        super(WrappingSprite, self).__init__(img, x, y, batch=batch)
+        self.collision_radius = self.image.width / COLLISION_RESOLUTION / 2 
 
     def update(self, dt):
-        self.x += self.dx * dt
-        self.y += self.dy * dt
-        self.heading += self.heading_speed * dt
+        x = self.x + self.dx * dt
+        y = self.y + self.dy * dt
+        rotation = self.rotation + self.rotation_speed * dt
         
-        self.x = wrap(self.x, ARENA_WIDTH)
-        self.y = wrap(self.y, ARENA_HEIGHT)
-        self.heading = wrap(self.heading, 360.)
+        self.x = wrap(x, ARENA_WIDTH)
+        self.y = wrap(y, ARENA_HEIGHT)
+        self.rotation = wrap(rotation, 360.)
 
     def collision_cells(self):
         '''Generate a sequence of (x, y) cells this object covers,
@@ -197,43 +138,21 @@ class WrappingObject(object):
             for x in range(cellx - radius, cellx + radius + 1):
                 yield x, y
 
-    def draw_at(self, x, y):
-        glLoadIdentity()
-        glTranslatef(x, y, 0)
-        glRotatef(self.heading, 0, 0, 1)
-        self.img.blit(-self.img.width/2, -self.img.height/2, 0)
-
-    def draw(self):
-        x_positions = [self.x]
-        y_positions = [self.y]
-
-        if self.x < self.img.width:
-            x_positions.append(self.x + ARENA_WIDTH)
-        if self.x > ARENA_WIDTH - self.img.width:
-            x_positions.append(self.x - ARENA_WIDTH)
-        if self.y < self.img.height:
-            y_positions.append(self.y + ARENA_HEIGHT)
-        if self.y > ARENA_HEIGHT - self.img.height:
-            y_positions.append(self.y - ARENA_HEIGHT)
-
-        for x in x_positions:
-            for y in y_positions:
-                self.draw_at(x, y)
-
 class AsteroidSize(object):
     def __init__(self, filename, points):
-        self.img = image.load(res(filename))
+        self.img = resource.image(filename)
+        center_anchor(self.img)
         self.next_size = None
         self.points = points
 
-class Asteroid(WrappingObject):
-    def __init__(self, x, y, size):
-        super(Asteroid, self).__init__(x, y, size.img)
+class Asteroid(WrappingSprite):
+    def __init__(self, size, x, y, batch=None):
+        super(Asteroid, self).__init__(size.img, x, y, batch=batch)
         self.dx = (random.random() - 0.5) * MAX_ASTEROID_SPEED
         self.dy = (random.random() - 0.5) * MAX_ASTEROID_SPEED
         self.size = size
-        self.heading = random.random() * 360.
-        self.heading_speed = (random.random() - 0.5) * MAX_ASTEROID_SPIN_SPEED
+        self.rotation = random.random() * 360.
+        self.rotation_speed = (random.random() - 0.5) * MAX_ASTEROID_SPIN_SPEED
         self.hit = False
 
     def destroy(self):
@@ -245,13 +164,17 @@ class Asteroid(WrappingObject):
         if next_size:
             # Spawn debris
             for i in range(ASTEROID_DEBRIS_COUNT):
-                asteroids.append(Asteroid(self.x, self.y, next_size))
+                asteroids.append(Asteroid(next_size, self.x, self.y,
+                                          batch=self.batch))
 
+        self.delete()
         asteroids.remove(self)
 
-class Player(WrappingObject, key.KeyStateHandler):
-    def __init__(self, img):
-        super(Player, self).__init__(ARENA_WIDTH / 2, ARENA_HEIGHT / 2, img)
+class Player(WrappingSprite, key.KeyStateHandler):
+    def __init__(self, img, batch=None):
+        super(Player, self).__init__(img, ARENA_WIDTH / 2, ARENA_HEIGHT / 2,
+            batch=batch)
+        center_anchor(img)
         self.reset()
 
     def reset(self):
@@ -259,7 +182,7 @@ class Player(WrappingObject, key.KeyStateHandler):
         self.y = ARENA_HEIGHT / 2
         self.dx = 0
         self.dy = 0
-        self.heading = 0
+        self.rotation = 0
         self.fire_timeout = 0
         self.hit = False
         self.invincible = True
@@ -269,20 +192,20 @@ class Player(WrappingObject, key.KeyStateHandler):
         self.flash_visible = False
 
     def update(self, dt):
-        # Update heading
+        # Update rotation
         if self[key.LEFT]:
-            self.heading += PLAYER_SPIN_SPEED * dt
+            self.rotation -= PLAYER_SPIN_SPEED * dt
         if self[key.RIGHT]:
-            self.heading -= PLAYER_SPIN_SPEED * dt
+            self.rotation += PLAYER_SPIN_SPEED * dt
 
         # Get x/y components of orientation
-        heading_x = math.cos(to_radians(self.heading))
-        heading_y = math.sin(to_radians(self.heading))
+        rotation_x = math.cos(to_radians(-self.rotation))
+        rotation_y = math.sin(to_radians(-self.rotation))
 
         # Update velocity
         if self[key.UP]:
-            self.dx += PLAYER_ACCEL * heading_x * dt
-            self.dy += PLAYER_ACCEL * heading_y * dt
+            self.dx += PLAYER_ACCEL * rotation_x * dt
+            self.dy += PLAYER_ACCEL * rotation_y * dt
 
         # Update position
         super(Player, self).update(dt)
@@ -296,8 +219,8 @@ class Player(WrappingObject, key.KeyStateHandler):
             # ship were bigger, or if bullets moved slower we'd adjust this
             # based on the orientation of the ship.
             bullets.append(Bullet(self.x, self.y, 
-                                  heading_x * BULLET_SPEED,
-                                  heading_y * BULLET_SPEED))
+                                  rotation_x * BULLET_SPEED,
+                                  rotation_y * BULLET_SPEED, batch=batch))
 
             if enable_sound:
                 bullet_sound.play()
@@ -311,33 +234,34 @@ class Player(WrappingObject, key.KeyStateHandler):
         else:
             self.flash_visible = True
 
-    def draw(self):
-        if self.visible and self.flash_visible:
-            super(Player, self).draw()
+        self.opacity = (self.visible and self.flash_visible) and 255 or 0
 
-class Bullet(object):
-    __slots__ = ['x', 'y', 'dx', 'dy', 'hit']
-
-    def __init__(self, x, y, dx, dy):
-        self.x = x
-        self.y = y
+class MovingSprite(pyglet.sprite.Sprite):
+    def __init__(self, image, x, y, dx, dy, batch=None):
+        super(MovingSprite, self).__init__(image, x, y, batch=batch)
         self.dx = dx
         self.dy = dy
 
     def update(self, dt):
-        # Bullets have a much simpler update method than the other objects as
-        # they are destroyed when the reach the edge of the arena rather than
-        # wrapping.
         self.x += self.dx * dt
         self.y += self.dy * dt
 
-    def in_bounds(self):
-        return (self.x >= 0 and self.x < ARENA_WIDTH and
-                self.y >= 0 and self.y < ARENA_HEIGHT)
+class Bullet(MovingSprite):
+    def __init__(self, x, y, dx, dy, batch=None):
+        super(Bullet, self).__init__(bullet_image, x, y, dx, dy, batch=batch)
 
-    def draw(self):
-        bullet_image.blit(self.x - bullet_image.width / 2, 
-                          self.y - bullet_image.height / 2, 0)
+    def update(self, dt):
+        self.x += self.dx * dt
+        self.y += self.dy * dt
+        if not (self.x >= 0 and self.x < ARENA_WIDTH and
+                self.y >= 0 and self.y < ARENA_HEIGHT):
+            self.delete()
+            bullets.remove(self)
+
+class EffectSprite(MovingSprite):
+    def on_animation_end(self):
+        self.delete()
+        animations.remove(self)
 
 class Starfield(object):
     def __init__(self, img):
@@ -376,16 +300,18 @@ class Overlay(object):
 
 class Banner(Overlay):
     def __init__(self, label, dismiss_func=None, timeout=None):
-        self.text = font.Text(create_font(48),
-                              label,
-                              x = ARENA_WIDTH / 2, y = ARENA_HEIGHT / 2,
-                              halign=font.Text.CENTER,
-                              valign=font.Text.CENTER)
+        self.text = pyglet.text.Label(label,
+                                      font_name=FONT_NAME,
+                                      font_size=36,
+                                      x=ARENA_WIDTH / 2, 
+                                      y=ARENA_HEIGHT / 2,
+                                      anchor_x='center',
+                                      anchor_y='center')
 
         self.dismiss_func = dismiss_func
         self.timeout = timeout
         if timeout and dismiss_func:
-            clock.schedule_once(dismiss_func, timeout)
+            pyglet.clock.schedule_once(dismiss_func, timeout)
 
     def draw(self):
         self.text.draw()
@@ -398,11 +324,13 @@ class Banner(Overlay):
 class Menu(Overlay):
     def __init__(self, title):
         self.items = []
-        self.title_text = font.Text(create_font(48),
-                                    title, 
-                                    x=ARENA_WIDTH / 2, y=350,
-                                    halign=font.Text.CENTER,
-                                    valign=font.Text.CENTER)
+        self.title_text = pyglet.text.Label(title, 
+                                            font_name=FONT_NAME,
+                                            font_size=36,
+                                            x=ARENA_WIDTH // 2, 
+                                            y=350,
+                                            anchor_x='center',
+                                            anchor_y='center')
 
     def reset(self):
         self.selected_index = 0
@@ -432,42 +360,39 @@ class MenuItem(object):
 
     def __init__(self, label, y, activate_func):
         self.y = y
-        self.text = font.Text(create_font(18),
-                              label,
-                              x=ARENA_WIDTH / 2, y=y,
-                              halign=font.Text.CENTER,
-                              valign=font.Text.CENTER)
+        self.text = pyglet.text.Label(label,
+                                      font_name=FONT_NAME,
+                                      font_size=14,
+                                      x=ARENA_WIDTH // 2, 
+                                      y=y,
+                                      anchor_x='center',
+                                      anchor_y='center')
         self.activate_func = activate_func
 
     def draw_pointer(self, x, y, color, flip=False):
-        if flip:
-            # Use the texture matrix to flip the image horizontally
-            glMatrixMode(GL_TEXTURE)
-            glPushMatrix()
-            glScalef(-1, 1, 1)
-
         # Tint the pointer image to a color
         glPushAttrib(GL_CURRENT_BIT)
         glColor3f(*color)
-        pointer_image.blit(x, y)
-        glPopAttrib()
-
         if flip:
-            glPopMatrix()
-            glMatrixMode(GL_MODELVIEW)
+            pointer_image_flip.blit(x, y)
+        else:
+            pointer_image.blit(x, y)
+        glPopAttrib()
 
     def draw(self, selected):
         self.text.draw()
 
         if selected:
             self.draw_pointer(
-                self.text.x - self.text.width / 2 - pointer_image.width,
-                self.y - pointer_image.height / 2, 
+                self.text.x - self.text.content_width / 2 - 
+                    pointer_image.width / 2,
+                self.y, 
                 self.pointer_color,
                 self.inverted_pointers)
             self.draw_pointer(
-                self.text.x + self.text.width / 2,
-                self.y - pointer_image.height / 2,
+                self.text.x + self.text.content_width / 2 + 
+                    pointer_image.width / 2,
+                self.y,
                 self.pointer_color,
                 not self.inverted_pointers)
 
@@ -507,15 +432,15 @@ class DifficultyMenuItem(MenuItem):
 
     def get_label(self):
         if difficulty == 0:
-            return 'DIFFICULTY: PEBBLES'
+            return 'Difficulty: Pebbles'
         elif difficulty == 1:
-            return 'DIFFICULTY: STONES'
+            return 'Difficulty: Stones'
         elif difficulty == 2:
-            return 'DIFFICULTY: ASTEROIDS'
+            return 'Difficulty: Asteroids'
         elif difficulty == 3:
-            return 'DIFFICULTY: METEORS'
+            return 'Difficulty: Meteors'
         else:
-            return 'DIFFICULTY: %d' % difficulty
+            return 'Difficulty: %d' % difficulty
 
     def on_key_press(self, symbol, modifiers):
         global difficulty
@@ -568,11 +493,13 @@ class InstructionsMenu(Menu):
         self.items.append(MenuItem('Ok', 50, begin_main_menu))
         self.reset()
 
-        self.instruction_text = font.Text(create_font(18),
-                                          INSTRUCTIONS,
-                                          x=20, y=300,
-                                          width=ARENA_WIDTH - 40,
-                                          valign=font.Text.TOP)
+        self.instruction_text = pyglet.text.Label(INSTRUCTIONS,
+                                                  font_name=FONT_NAME,
+                                                  font_size=14,
+                                                  x=20, y=300,
+                                                  width=ARENA_WIDTH - 40,
+                                                  anchor_y='top',
+                                                  multiline=True)
 
     def draw(self):
         super(InstructionsMenu, self).draw()
@@ -663,20 +590,26 @@ def begin_round(*args):
     for i in range(INITIAL_ASTEROIDS[difficulty]):
         x = random.random() * ARENA_WIDTH
         y = random.random() * ARENA_HEIGHT
-        asteroids.append(Asteroid(x, y, asteroid_sizes[-1]))
+        asteroids.append(Asteroid(asteroid_sizes[-1], x, y, wrapping_batch))
+
+    for bullet in bullets:
+        bullet.delete()
+
+    for animation in animations:
+        animation.delete()
 
     bullets = []
     animations = []
     in_game = True
     set_overlay(None)
-    clock.schedule_once(begin_play, BEGIN_PLAY_DELAY)
+    pyglet.clock.schedule_once(begin_play, BEGIN_PLAY_DELAY)
 
 def begin_play(*args):
     player.invincible = False
 
 def begin_life(*args):
     player.reset()
-    clock.schedule_once(begin_play, BEGIN_PLAY_DELAY)
+    pyglet.clock.schedule_once(begin_play, BEGIN_PLAY_DELAY)
     
 def life_lost(*args):
     global player_lives
@@ -706,15 +639,15 @@ def end_game():
     paused = False
     in_game = False
     player.invincible = True
-    clock.unschedule(life_lost)
-    clock.unschedule(begin_play)
+    pyglet.clock.unschedule(life_lost)
+    pyglet.clock.unschedule(begin_play)
     begin_menu_background()
     set_overlay(MainMenu())
 
 def set_overlay(new_overlay):
     global overlay
     if overlay:
-        win.pop_handlers()
+        win.remove_handlers(overlay)
     overlay = new_overlay
     if overlay:
         win.push_handlers(overlay)
@@ -730,7 +663,13 @@ def begin_menu_background():
     for i in range(11):
         x = random.random() * ARENA_WIDTH
         y = random.random() * ARENA_HEIGHT
-        asteroids.append(Asteroid(x, y, asteroid_sizes[i // 4]))
+        asteroids.append(Asteroid(asteroid_sizes[i // 4], x, y, wrapping_batch))
+
+    for bullet in bullets:
+        bullet.delete()
+
+    for animation in animations:
+        animation.delete()
 
     bullets = []
     animations = []
@@ -743,27 +682,84 @@ def begin_clear_background():
     global bullets
     global animations
 
+    for bullet in bullets:
+        bullet.delete()
+
+    for animation in animations:
+        animation.delete()
+
     asteroids = []
     bullets = []
     animations = []
     player.visible = False
 
+# --------------------------------------------------------------------------
+# Create window
+# --------------------------------------------------------------------------
+
+win = pyglet.window.Window(ARENA_WIDTH, ARENA_HEIGHT, caption='Astraea')
+
+@win.event
+def on_key_press(symbol, modifiers):
+    # Overrides default Escape key behaviour
+    if symbol == KEY_PAUSE and in_game:
+        if not paused:
+            pause_game()
+        else:
+            resume_game()
+        return True
+    elif symbol == key.ESCAPE:
+        sys.exit()
+    return pyglet.event.EVENT_HANDLED
+
+@win.event
+def on_draw():
+    glColor3f(1, 1, 1)
+
+    # Render
+    starfield.draw()
+
+    for (x, y) in ((0, ARENA_HEIGHT),   # Top
+                   (-ARENA_WIDTH, 0),   # Left
+                   (0, 0),              # Center
+                   (ARENA_WIDTH, 0),    # Right
+                   (0, -ARENA_HEIGHT)): # Bottom
+        glLoadIdentity()
+        glTranslatef(x, y, 0)
+        wrapping_batch.draw()
+
+    glLoadIdentity()
+    batch.draw()
+
+    glLoadIdentity()
+
+    if in_game:
+        # HUD ship lives
+        x = 10 + player.image.width // 2
+        for i in range(player_lives - 1):
+            player.image.blit(x, win.height - player.image.height // 2 - 10, 0)
+            x += player.image.width + 10
+        
+        # HUD score
+        score_text.text = str(score)
+        score_text.draw()
+
+    if overlay:
+        overlay.draw()
+
+    if show_fps:
+        fps_display.draw()
+
+
 # --------------------------------------------------------------------------
 # Load resources
 # --------------------------------------------------------------------------
 
-def res(filename):
-    frozen = getattr(sys, 'frozen', None)
-    if frozen in ('windows_exe', 'console_exe'):
-        res_dir = os.path.join(os.path.dirname(sys.executable), 'res')
-    elif frozen == 'macosx_app':
-        res_dir = os.path.join(os.environ['RESOURCEPATH'], 'res')
-    else:
-        res_dir = os.path.join(os.path.dirname(__file__), 'res')
-    return os.path.join(res_dir, filename)
+batch = pyglet.graphics.Batch()
+wrapping_batch = pyglet.graphics.Batch()
 
-def create_font(size):
-    return font.load('Saved By Zero', size, dpi=72)
+resource.path.append('res')
+resource.reindex()
 
 asteroid_sizes = [AsteroidSize('asteroid1.png', 100),
                   AsteroidSize('asteroid2.png', 50),
@@ -771,26 +767,42 @@ asteroid_sizes = [AsteroidSize('asteroid1.png', 100),
 for small, big in map(None, asteroid_sizes[:-1], asteroid_sizes[1:]):
     big.next_size = small
 
-bullet_image = image.load(res('bullet.png'))
+bullet_image = resource.image('bullet.png')
+center_anchor(bullet_image)
 
-smoke_images_image = image.load(res('smoke.png'))
-smoke_images = image.ImageGrid(smoke_images_image, 1, 8)
+smoke_images_image = resource.image('smoke.png')
+smoke_images = pyglet.image.ImageGrid(smoke_images_image, 1, 8)
+map(center_anchor, smoke_images)
+smoke_animation = \
+    pyglet.image.Animation.from_image_sequence(smoke_images,
+                                               SMOKE_ANIMATION_PERIOD,
+                                               loop=False)
 
-explosion_images_image = image.load(res('explosion.png'))
-explosion_images = image.ImageGrid(explosion_images_image, 2, 8)
+explosion_images_image = resource.image('explosion.png')
+explosion_images = pyglet.image.ImageGrid(explosion_images_image, 2, 8)
+explosion_images = explosion_images.get_texture_sequence()
+map(center_anchor, explosion_images)
+explosion_animation = \
+    pyglet.image.Animation.from_image_sequence(explosion_images,
+                                               EXPLOSION_ANIMATION_PERIOD,
+                                               loop=False)
 
-pointer_image = image.load(res('pointer.png'))
+pointer_image = resource.image('pointer.png')
+pointer_image.anchor_x = pointer_image.width // 2
+pointer_image.anchor_y = pointer_image.height // 2
+pointer_image_flip = resource.image('pointer.png', flip_x=True)
 
-explosion_sound = media.load(res('explosion.wav'), streaming=False)
-bullet_sound = media.load(res('bullet.wav'), streaming=False)
+explosion_sound = resource.media('explosion.wav', streaming=False)
+bullet_sound = resource.media('bullet.wav', streaming=False)
+
+starfield = Starfield(resource.image('starfield.jpg'))
+player = Player(resource.image('ship.png'), wrapping_batch)
+win.push_handlers(player)
 
 # --------------------------------------------------------------------------
 # Global game state vars
 # --------------------------------------------------------------------------
 
-starfield = Starfield(image.load(res('starfield.jpg')))
-player = Player(image.load(res('ship.png')))
-
 overlay = None
 in_game = False
 paused = False
@@ -800,123 +812,76 @@ difficulty = 2
 show_fps = False
 enable_sound = True
 
-if __name__ == '__main__':
-    win = window.Window(ARENA_WIDTH, ARENA_HEIGHT, caption='Astraea')
+score_text = pyglet.text.Label('',
+                               font_name=FONT_NAME,
+                               font_size=18,
+                               x=ARENA_WIDTH - 10, 
+                               y=ARENA_HEIGHT - 10,
+                               anchor_x='right',
+                               anchor_y='top')
 
-    # Override default Escape key behaviour
-    def on_key_press(symbol, modifiers):
-        if symbol == KEY_PAUSE and in_game:
-            if not paused:
-                pause_game()
-            else:
-                resume_game()
-            return True
-        elif symbol == key.ESCAPE:
-            sys.exit()
-    win.on_key_press = on_key_press
+fps_display = pyglet.clock.ClockDisplay(font=pyglet.font.load(FONT_NAME, 24))
 
-    win.push_handlers(player)
+bullets = []
+animations = []
 
-    # Now that we have a GL context, construct textures.
-    smoke_images = smoke_images.texture_sequence
-    explosion_images = explosion_images.texture_sequence
+# --------------------------------------------------------------------------
+# Game update
+# --------------------------------------------------------------------------
 
-    font.add_file(res('saved_by_zero.ttf'))
+def update(dt):
+    if overlay:
+        overlay.update(dt)
 
-    score_text = font.Text(create_font(24),
-                           '',
-                           x = ARENA_WIDTH - 10, y = ARENA_HEIGHT - 10,
-                           halign=font.Text.RIGHT, valign=font.Text.TOP)
-
-    glEnable(GL_BLEND)
-    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
-
-    begin_menu_background()
-    begin_main_menu()
-
-    fps_display = clock.ClockDisplay(font=create_font(36))
-
-    while not win.has_exit:
-        win.dispatch_events()
-        media.dispatch_events()
-
-        # Update
-        dt = clock.tick()
-        if overlay:
-            overlay.update(dt)
-
-        if not paused:
-            starfield.update(dt)
-            player.update(dt)
-            for asteroid in asteroids:
-                asteroid.update(dt)
-            for bullet in bullets:
-                bullet.update(dt)
-            for animation in animations:
-                animation.update(dt)
-
-        # Destroy bullets that have left the arena
-        bullets = [bullet for bullet in bullets if bullet.in_bounds()]
-
-        # Cancel animations that are finished
-        animations = [ani for ani in animations if not ani.finished]
-
-        if not player.invincible:
-            # Collide bullets and player with asteroids
-            check_collisions()
-
-            # Destroy asteroids that were hit
-            for asteroid in [a for a in asteroids if a.hit]:
-                animations.append(Animation(asteroid.x, asteroid.y, 
-                                            asteroid.dx, asteroid.dy,
-                                            smoke_images, 
-                                            SMOKE_ANIMATION_PERIOD))
-                asteroid.destroy()
-                if enable_sound:
-                    explosion_sound.play()
-
-            # Check if the player was hit 
-            if player.hit:
-                animations.append(Animation(player.x, player.y,
-                                            player.dx, player.dy,
-                                            explosion_images,
-                                            EXPLOSION_ANIMATION_PERIOD))
-                player.invincible = True
-                player.visible = False
-                clock.schedule_once(life_lost, LIFE_LOST_DELAY)
-
-            # Check if the area is clear
-            if not asteroids:
-                next_round()
-
-        # Render
-        starfield.draw()
+    if not paused:
+        starfield.update(dt)
 
+        player.update(dt)
         for asteroid in asteroids:
-            asteroid.draw()
-        glLoadIdentity()
-        for bullet in bullets:
-            bullet.draw()
-        player.draw()
-        glLoadIdentity()
-        for animation in animations:
-            animation.draw()
+            asteroid.update(dt)
+        for bullet in bullets[:]:
+            bullet.update(dt)
+        for animation in animations[:]:
+            animation.update(dt)
 
-        if in_game:
-            # HUD ship lives
-            x = 10
-            for i in range(player_lives - 1):
-                player.img.blit(x, win.height - player.img.height - 10, 0)
-                x += player.img.width + 10
-            
-            # HUD score
-            score_text.text = str(score)
-            score_text.draw()
 
-        if overlay:
-            overlay.draw()
+    if not player.invincible:
+        # Collide bullets and player with asteroids
+        check_collisions()
 
-        if show_fps:
-            fps_display.draw()
+        # Destroy asteroids that were hit
+        for asteroid in [a for a in asteroids if a.hit]:
+            animations.append(EffectSprite(smoke_animation,
+                                           asteroid.x, asteroid.y, 
+                                           asteroid.dx, asteroid.dy,
+                                           batch=batch))
+            asteroid.destroy()
+            if enable_sound:
+                explosion_sound.play()
 
-        win.flip()
+        # Check if the player was hit 
+        if player.hit:
+            animations.append(EffectSprite(explosion_animation,
+                                           player.x, player.y,
+                                           player.dx, player.dy, 
+                                           batch=batch))
+            player.invincible = True
+            player.visible = False
+            pyglet.clock.schedule_once(life_lost, LIFE_LOST_DELAY)
+
+        # Check if the area is clear
+        if not asteroids:
+            next_round()
+pyglet.clock.schedule_interval(update, 1/60.)
+
+# --------------------------------------------------------------------------
+# Start game
+# --------------------------------------------------------------------------
+
+glEnable(GL_BLEND)
+glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
+
+begin_menu_background()
+begin_main_menu()
+
+pyglet.app.run()
