@@ -32,7 +32,7 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 # ----------------------------------------------------------------------------
-# $Id: soundspace.py 1579 2008-01-15 14:47:19Z Alex.Holkner $
+# $Id: soundspace.py 2529 2009-10-12 17:39:44Z benjamin.coder.smith@gmail.com $
 
 import math
 import os
@@ -40,10 +40,14 @@ import os
 from pyglet.gl import *
 from pyglet import font
 from pyglet import media
+from pyglet import resource
 from pyglet import window
 
 import reader
 
+resource.path.append('res')
+resource.reindex()
+
 # Check for AVbin
 try:
     from pyglet.media import avbin
@@ -98,10 +102,10 @@ class Handle(object):
         return self
 
     def on_mouse_press(self, x, y, button, modifiers):
-        self.win.pop_handlers()
+        self.win.remove_handlers(self)
 
     def on_mouse_release(self, x, y, button, modifiers):
-        self.win.pop_handlers()
+        self.win.remove_handlers(self)
 
 class LabelHandle(Handle):
     def __init__(self, player):
@@ -347,7 +351,7 @@ class MoreHandle(Handle):
             if handle.hit_test(x, y, z):
                 return 
         self.win.set_more_player_handles(None)
-        self.win.pop_handlers()
+        self.win.remove_handlers(self)
         self.open = False
 
     def on_mouse_release(self, x, y, button, modifiers):
@@ -583,18 +587,15 @@ class PanView(object):
         self.win = window
 
     def on_mouse_release(self, x, y, button, modifiers):
-        self.win.pop_handlers()
+        self.win.remove_handlers(self)
 
     def on_mouse_press(self, x, y, button, modifiers):
-        self.win.pop_handlers()
+        self.win.remove_handlers(self)
 
     def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
         self.win.tx += dx
         self.win.ty += dy
 
-def res(filename):
-    return os.path.join(os.path.dirname(__file__), 'res', filename)
-
 if __name__ == '__main__':
     # We swap Y and Z, moving to left-handed system
     media.listener.up_orientation = (0, -1, 0)
@@ -606,7 +607,7 @@ if __name__ == '__main__':
 
     w = SoundSpaceWindow()
     r = reader.SpaceReader(w)
-    r.read(res('space.txt'))
+    r.read(resource.file('space.txt'))
     for player in w.players:
         player.play()
         pass
