@@ -2,14 +2,14 @@
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions 
+# modification, are permitted provided that the following conditions
 # are met:
 #
 #  * Redistributions of source code must retain the above copyright
 #    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright 
+#  * Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in
 #    the documentation and/or other materials provided with the
 #    distribution.
@@ -40,22 +40,22 @@ Three layout classes are provided:
 `TextLayout`
     The entire document is laid out before it is rendered.  The layout will
     be grouped with other layouts in the same batch (allowing for efficient
-    rendering of multiple layouts).  
-    
+    rendering of multiple layouts).
+
     Any change to the layout or document,
     and even querying some properties, will cause the entire document
     to be laid out again.
 
 `ScrollableTextLayout`
     Based on `TextLayout`.
-    
+
     A separate group is used for layout which crops the contents of the
     layout to the layout rectangle.  Additionally, the contents of the
     layout can be "scrolled" within that rectangle with the ``view_x`` and
     ``view_y`` properties.
 
 `IncrementalTextLayout`
-    Based on `ScrollableTextLayout`.    
+    Based on `ScrollableTextLayout`.
 
     When the layout or document are modified, only the affected regions
     are laid out again.  This permits efficient interactive editing and
@@ -77,13 +77,13 @@ Style attributes
 ================
 
 The following character style attribute names are recognised by the layout
-classes.  Data types and units are as specified.  
+classes.  Data types and units are as specified.
 
 Where an attribute is marked "as a distance" the value is assumed to be
 in pixels if given as an int or float, otherwise a string of the form
 ``"0u"`` is required, where ``0`` is the distance and ``u`` is the unit; one
 of ``"px"`` (pixels), ``"pt"`` (points), ``"pc"`` (picas), ``"cm"``
-(centimeters), ``"mm"`` (millimeters) or ``"in"`` (inches).  For example, 
+(centimeters), ``"mm"`` (millimeters) or ``"in"`` (inches).  For example,
 ``"14pt"`` is the distance covering 14 points, which at the default DPI of 96
 is 18 pixels.
 
@@ -118,7 +118,7 @@ entire paragraph, otherwise results are undefined.
 ``align``
     ``left`` (default), ``center`` or ``right``.
 ``indent``
-    Additional horizontal space to insert before the first glyph of the 
+    Additional horizontal space to insert before the first glyph of the
     first line of a paragraph, as a distance.
 ``leading``
     Additional space to insert between consecutive lines within a paragraph,
@@ -151,16 +151,16 @@ document; they will be ignored by the built-in text classes.
 __docformat__ = 'restructuredtext'
 __version__ = '$Id: $'
 
-import math
 import re
 import sys
 
 from pyglet.gl import *
 from pyglet import event
 from pyglet import graphics
-from pyglet.text import document
 from pyglet.text import runlist
 
+from pyglet.font.base import _grapheme_break
+
 _is_epydoc = hasattr(sys, 'is_epydoc') and sys.is_epydoc
 
 _distance_re = re.compile(r'([-0-9.]+)([a-zA-Z]+)')
@@ -203,7 +203,7 @@ class _Line(object):
     margin_right = 0
 
     length = 0
-        
+
     ascent = 0
     descent = 0
     width = 0
@@ -241,7 +241,7 @@ class _LayoutContext(object):
         self.colors_iter = colors_iter
         underline_iter = document.get_style_runs('underline')
         self.decoration_iter = runlist.ZipRunIterator(
-            (background_iter, 
+            (background_iter,
              underline_iter))
         self.baseline_iter = runlist.FilteredRunIterator(
             document.get_style_runs('baseline'),
@@ -313,7 +313,7 @@ class _GlyphBox(_AbstractBox):
         self.owner = owner
         self.font = font
         self.glyphs = glyphs
-        self.advance = advance 
+        self.advance = advance
 
     def place(self, layout, i, x, y, context):
         assert self.glyphs
@@ -341,7 +341,7 @@ class _GlyphBox(_AbstractBox):
                 t = glyph.tex_coords
                 tex_coords.extend(t)
                 x1 += glyph.advance
-        
+
         # Text color
         colors = []
         for start, end, color in context.colors_iter.ranges(i, i+n_glyphs):
@@ -349,7 +349,7 @@ class _GlyphBox(_AbstractBox):
                 color = (0, 0, 0, 255)
             colors.extend(color * ((end - start) * 4))
 
-        list = layout.batch.add(n_glyphs * 4, GL_QUADS, group, 
+        list = layout.batch.add(n_glyphs * 4, GL_QUADS, group,
             ('v2f/dynamic', vertices),
             ('t3f/dynamic', tex_coords),
             ('c4B/dynamic', colors))
@@ -386,7 +386,7 @@ class _GlyphBox(_AbstractBox):
                 underline_colors.extend(underline * 2)
 
             x1 = x2
-            
+
         if background_vertices:
             background_list = layout.batch.add(
                 len(background_vertices) // 2, GL_QUADS,
@@ -432,7 +432,7 @@ class _GlyphBox(_AbstractBox):
 class _InlineElementBox(_AbstractBox):
     def __init__(self, element):
         '''Create a glyph run holding a single element.
-        '''        
+        '''
         super(_InlineElementBox, self).__init__(
             element.ascent, element.descent, element.advance, 1)
         self.element = element
@@ -460,7 +460,7 @@ class _InlineElementBox(_AbstractBox):
             return 0
         else:
             return 1
-    
+
     def __repr__(self):
         return '_InlineElementBox(%r)' % self.element
 
@@ -506,9 +506,9 @@ class _InvalidRange(object):
 # top_group                     [Scrollable]TextLayoutGroup(Group)
 #   background_group            OrderedGroup(0)
 #   foreground_group            TextLayoutForegroundGroup(OrderedGroup(1))
-#     [font textures]           TextLayoutTextureGroup(Group)  
-#     [...]                     TextLayoutTextureGroup(Group)  
-#   foreground_decoration_group 
+#     [font textures]           TextLayoutTextureGroup(Group)
+#     [...]                     TextLayoutTextureGroup(Group)
+#   foreground_decoration_group
 #                       TextLayoutForegroundDecorationGroup(OrderedGroup(2))
 
 class TextLayoutGroup(graphics.Group):
@@ -525,7 +525,7 @@ class TextLayoutGroup(graphics.Group):
 
     def unset_state(self):
         glPopAttrib()
-        
+
 class ScrollableTextLayoutGroup(graphics.Group):
     '''Top-level rendering group for `ScrollableTextLayout`.
 
@@ -642,7 +642,7 @@ class TextLayoutForegroundGroup(graphics.OrderedGroup):
     def set_state(self):
         glEnable(GL_TEXTURE_2D)
 
-    # unset_state not needed, as parent group will pop enable bit 
+    # unset_state not needed, as parent group will pop enable bit
 
 class TextLayoutForegroundDecorationGroup(graphics.OrderedGroup):
     '''Rendering group for decorative elements (e.g., glyph underlines) in all
@@ -653,7 +653,7 @@ class TextLayoutForegroundDecorationGroup(graphics.OrderedGroup):
     def set_state(self):
         glDisable(GL_TEXTURE_2D)
 
-    # unset_state not needed, as parent group will pop enable bit 
+    # unset_state not needed, as parent group will pop enable bit
 
 class TextLayoutTextureGroup(graphics.Group):
     '''Rendering group for a glyph texture in all text layouts.
@@ -677,12 +677,12 @@ class TextLayoutTextureGroup(graphics.Group):
         return hash((self.texture.id, self.parent))
 
     def __eq__(self, other):
-        return (self.__class__ is other.__class__ and 
+        return (self.__class__ is other.__class__ and
                 self.texture.id == other.texture.id and
                 self.parent is other.parent)
 
     def __repr__(self):
-        return '%s(%d, %r)' % (self.__class__.__name__, 
+        return '%s(%d, %r)' % (self.__class__.__name__,
                                self.texture.id,
                                self.parent)
 
@@ -703,10 +703,10 @@ class TextLayout(object):
 
     :Ivariables:
         `content_width` : int
-            Calculuated width of the text in the layout.  This may overflow
+            Calculated width of the text in the layout.  This may overflow
             the desired width if word-wrapping failed.
         `content_height` : int
-            Calculuated height of the text in the layout.
+            Calculated height of the text in the layout.
         `top_group` : `Group`
             Top-level rendering group.
         `background_group` : `Group`
@@ -729,6 +729,7 @@ class TextLayout(object):
 
     _update_enabled = True
     _own_batch = False
+    _origin_layout = False  # Lay out relative to origin?  Otherwise to box.
 
     def __init__(self, document, width=None, height=None,
                  multiline=False, dpi=None, batch=None, group=None):
@@ -750,7 +751,7 @@ class TextLayout(object):
                 Optional graphics batch to add this layout to.
             `group` : `Group`
                 Optional rendering group to parent all groups this text layout
-                uses.  Note that layouts with different 
+                uses.  Note that layouts with different
                 rendered simultaneously in a batch.
 
         '''
@@ -776,7 +777,7 @@ class TextLayout(object):
         if dpi is None:
             dpi = 96
         self._dpi = dpi
-        self.document = document       
+        self.document = document
 
     def _parse_distance(self, distance):
         if distance is None:
@@ -805,7 +806,7 @@ class TextLayout(object):
         self._update_enabled = True
         self._update()
 
-    dpi = property(lambda self: self._dpi, 
+    dpi = property(lambda self: self._dpi,
                    doc='''Get DPI used by this layout.
 
     Read-only.
@@ -854,7 +855,7 @@ class TextLayout(object):
         document.push_handlers(self)
         self._document = document
         self._init_document()
-    
+
     document = property(_get_document, _set_document,
                        '''Document to display.
 
@@ -869,7 +870,7 @@ class TextLayout(object):
         glyphs = self._get_glyphs()
         owner_runs = runlist.RunList(len_text, None)
         self._get_owner_runs(owner_runs, glyphs, 0, len_text)
-        lines = [line for line in self._flow_glyphs(glyphs, owner_runs, 
+        lines = [line for line in self._flow_glyphs(glyphs, owner_runs,
                                                     0, len_text)]
         self.content_width = 0
         self._flow_lines(lines, 0, len(lines))
@@ -886,7 +887,7 @@ class TextLayout(object):
         self._vertex_lists = []
         self._boxes = []
         self.groups.clear()
-        
+
         if not self._document or not self._document.text:
             return
 
@@ -895,13 +896,16 @@ class TextLayout(object):
         colors_iter = self._document.get_style_runs('color')
         background_iter = self._document.get_style_runs('background_color')
 
-        left = self._get_left()
-        top = self._get_top(lines)
-        
-        context = _StaticLayoutContext(self, self._document, 
+        if self._origin_layout:
+            left = top = 0
+        else:
+            left = self._get_left()
+            top = self._get_top(lines)
+
+        context = _StaticLayoutContext(self, self._document,
                                        colors_iter, background_iter)
         for line in lines:
-            self._create_vertex_lists(left + line.x, top + line.y, 
+            self._create_vertex_lists(left + line.x, top + line.y,
                                       line.start, line.boxes, context)
 
     def _get_left(self):
@@ -951,7 +955,7 @@ class TextLayout(object):
         else:
             assert False, 'Invalid anchor_y'
 
- 
+
     def _init_document(self):
         self._update()
 
@@ -1004,12 +1008,12 @@ class TextLayout(object):
                 owner_runs.set_run(run_start, i + start, owner)
                 owner = glyph.owner
                 run_start = i + start
-        owner_runs.set_run(run_start, end, owner)    
+        owner_runs.set_run(run_start, end, owner)
 
     def _flow_glyphs(self, glyphs, owner_runs, start, end):
         # TODO change flow generator on self, avoiding this conditional.
         if not self._multiline:
-            for line in self._flow_glyphs_single_line(glyphs, owner_runs, 
+            for line in self._flow_glyphs_single_line(glyphs, owner_runs,
                                                       start, end):
                 yield line
         else:
@@ -1028,7 +1032,7 @@ class TextLayout(object):
 
         align_iterator = runlist.FilteredRunIterator(
             self._document.get_style_runs('align'),
-            lambda value: value in ('left', 'right', 'center'), 
+            lambda value: value in ('left', 'right', 'center'),
             'left')
         if self._width is None:
             wrap_iterator = runlist.ConstRunIterator(
@@ -1039,7 +1043,7 @@ class TextLayout(object):
                 lambda value: value in (True, False),
                 True)
         margin_left_iterator = runlist.FilteredRunIterator(
-            self._document.get_style_runs('margin_left'), 
+            self._document.get_style_runs('margin_left'),
             lambda value: value is not None, 0)
         margin_right_iterator = runlist.FilteredRunIterator(
             self._document.get_style_runs('margin_right'),
@@ -1053,7 +1057,7 @@ class TextLayout(object):
         tab_stops_iterator = runlist.FilteredRunIterator(
             self._document.get_style_runs('tab_stops'),
             lambda value: value is not None, [])
-        
+
         line = _Line(start)
         line.align = align_iterator[start]
         line.margin_left = self._parse_distance(margin_left_iterator[start])
@@ -1130,7 +1134,7 @@ class TextLayout(object):
                             tab = 50.
                             tab_stop = \
                               (((x + line.margin_left) // tab) + 1) * tab
-                        kern = int(tab_stop - x - line.margin_left - 
+                        kern = int(tab_stop - x - line.margin_left -
                                    glyph.advance)
 
                     owner_accum.append((kern, glyph))
@@ -1144,7 +1148,7 @@ class TextLayout(object):
 
                     x += glyph.advance + kern
                     index += 1
-                        
+
                     # The index at which the next line will begin (the
                     # current index, because this is the current best
                     # breakpoint).
@@ -1220,7 +1224,7 @@ class TextLayout(object):
                                 nokern = True
 
                             x = run_accum_width + owner_accum_width
-                            width = (self._width - 
+                            width = (self._width -
                                      line.margin_left - line.margin_right)
 
                     if isinstance(glyph, _AbstractBox):
@@ -1234,7 +1238,7 @@ class TextLayout(object):
                         line.margin_left += \
                             self._parse_distance(indent_iterator[next_start])
                         if width is not None:
-                            width = (self._width - 
+                            width = (self._width -
                                      line.margin_left - line.margin_right)
                     elif not new_line:
                         # If the glyph was any non-whitespace, non-newline
@@ -1259,7 +1263,7 @@ class TextLayout(object):
         # the final line.
         for run in run_accum:
             line.add_box(run)
-    
+
         if not line.boxes:
             # Empty line gets font's line-height
             if font is None:
@@ -1277,14 +1281,13 @@ class TextLayout(object):
             lambda value: value is not None, 0)
 
         line = _Line(start)
-        x = 0
         font = font_iterator[0]
 
         for start, end, owner in owner_iterator:
             font = font_iterator[start]
             width = 0
             owner_glyphs = []
-            for kern_start, kern_end, kern in kern_iterator.ranges(start, end): 
+            for kern_start, kern_end, kern in kern_iterator.ranges(start, end):
                 gs = glyphs[kern_start:kern_end]
                 width += sum([g.advance for g in gs])
                 width += kern * (kern_end - kern_start)
@@ -1295,14 +1298,14 @@ class TextLayout(object):
                     line.add_box(glyph)
             else:
                 line.add_box(_GlyphBox(owner, font, owner_glyphs, width))
-    
+
         if not line.boxes:
             line.ascent = font.ascent
             line.descent = font.descent
 
         line.paragraph_begin = line.paragraph_end = True
 
-        yield line 
+        yield line
 
     def _flow_lines(self, lines, start, end):
         margin_top_iterator = runlist.FilteredRunIterator(
@@ -1340,7 +1343,7 @@ class TextLayout(object):
                 leading = self._parse_distance(leading_iterator[line.start])
             else:
                 y -= leading
-            
+
             if line_spacing is None:
                 y -= line.ascent
             else:
@@ -1353,10 +1356,10 @@ class TextLayout(object):
             elif line.align == 'right':
                 line.x = self.width - line.margin_right - line.width
 
-            self.content_width = max(self.content_width, 
+            self.content_width = max(self.content_width,
                                      line.width + line.margin_left)
 
-            if line.y == y and line_index >= end: 
+            if line.y == y and line_index >= end:
                 # Early exit: all invalidated lines have been reflowed and the
                 # next line has no change (therefore subsequent lines do not
                 # need to be changed).
@@ -1373,7 +1376,7 @@ class TextLayout(object):
             self.content_height = -y
 
         return line_index
-        
+
     def _create_vertex_lists(self, x, y, i, boxes, context):
         for box in boxes:
             box.place(self, i, x, y, context)
@@ -1397,7 +1400,7 @@ class TextLayout(object):
     def _get_x(self):
         return self._x
 
-    x = property(_get_x, _set_x, 
+    x = property(_get_x, _set_x,
                  doc='''X coordinate of the layout.
 
     See also `anchor_x`.
@@ -1441,7 +1444,7 @@ class TextLayout(object):
 
     width = property(_get_width, _set_width,
                      doc='''Width of the layout.
-                     
+
     This property has no effect if `multiline` is False.
 
     :type: int
@@ -1457,7 +1460,7 @@ class TextLayout(object):
 
     height = property(_get_height, _set_height,
                       doc='''Height of the layout.
-                      
+
     :type: int
     ''')
 
@@ -1474,7 +1477,7 @@ class TextLayout(object):
 
     If multiline is False, newline and paragraph characters are ignored and
     text is not word-wrapped.
-                         
+
     :type: bool
     ''')
 
@@ -1488,7 +1491,7 @@ class TextLayout(object):
 
     anchor_x = property(_get_anchor_x, _set_anchor_x,
                       doc='''Horizontal anchor alignment.
-                      
+
     This property determines the meaning of the `x` coordinate.  It is one of
     the enumerants:
 
@@ -1516,7 +1519,7 @@ class TextLayout(object):
 
     anchor_y = property(_get_anchor_y, _set_anchor_y,
                       doc='''Vertical anchor alignment.
-                      
+
     This property determines the meaning of the `y` coordinate.  It is one of
     the enumerants:
 
@@ -1537,7 +1540,7 @@ class TextLayout(object):
     See also `content_valign`.
 
     :type: str
-    ''') 
+    ''')
 
     _content_valign = 'top'
     def _set_content_valign(self, content_valign):
@@ -1547,7 +1550,7 @@ class TextLayout(object):
     def _get_content_valign(self):
         return self._content_valign
 
-    content_valign = property(_get_content_valign, _set_content_valign, 
+    content_valign = property(_get_content_valign, _set_content_valign,
                               doc='''Vertical alignment of content within
     larger layout box.
 
@@ -1580,13 +1583,17 @@ class ScrollableTextLayout(TextLayout):
 
     Use `view_x` and `view_y` to scroll the text within the viewport.
     '''
+    _origin_layout = True
+
     def __init__(self, document, width, height, multiline=False, dpi=None,
                  batch=None, group=None):
         super(ScrollableTextLayout, self).__init__(
             document, width, height, multiline, dpi, batch, group)
+        self.top_group.width = self._width
+        self.top_group.height = self._height
 
     def _init_groups(self, group):
-        # Scrollable layout never shares group becauase of translation.   
+        # Scrollable layout never shares group becauase of translation.
         self.top_group = ScrollableTextLayoutGroup(group)
         self.background_group = graphics.OrderedGroup(0, self.top_group)
         self.foreground_group = TextLayoutForegroundGroup(1, self.top_group)
@@ -1612,10 +1619,9 @@ class ScrollableTextLayout(TextLayout):
     y = property(_get_y, _set_y)
 
     def _set_width(self, width):
-        self._width = width
-        self.top_group.width = width
+        super(ScrollableTextLayout, self)._set_width(width)
         self.top_group.left = self._get_left()
-        self._update()
+        self.top_group.width = self._width
 
     def _get_width(self):
         return self._width
@@ -1623,9 +1629,9 @@ class ScrollableTextLayout(TextLayout):
     width = property(_get_width, _set_width)
 
     def _set_height(self, height):
-        self._height = height
-        self.top_group.height = height
+        super(ScrollableTextLayout, self)._set_height(height)
         self.top_group.top = self._get_top(self._get_lines())
+        self.top_group.height = self._height
 
     def _get_height(self):
         return self._height
@@ -1635,7 +1641,7 @@ class ScrollableTextLayout(TextLayout):
     def _set_anchor_x(self, anchor_x):
         self._anchor_x = anchor_x
         self.top_group.left = self._get_left()
-        
+
     def _get_anchor_x(self):
         return self._anchor_x
 
@@ -1644,7 +1650,7 @@ class ScrollableTextLayout(TextLayout):
     def _set_anchor_y(self, anchor_y):
         self._anchor_y = anchor_y
         self.top_group.top = self._get_top(self._get_lines())
-        
+
     def _get_anchor_y(self):
         return self._anchor_y
 
@@ -1666,7 +1672,7 @@ class ScrollableTextLayout(TextLayout):
     side of the layout bounds.  A positive value causes the text to "scroll"
     to the right.  Values are automatically clipped into the range
     ``[0, content_width - width]``
-                      
+
     :type: int
     ''')
 
@@ -1683,8 +1689,8 @@ class ScrollableTextLayout(TextLayout):
 
     The initial value is 0, and the top of the text will touch the top of the
     layout bounds (unless the content height is less than the layout height,
-    in which case `content_valign` is used).  
-    
+    in which case `content_valign` is used).
+
     A negative value causes the text to "scroll" upwards.  Values outside of
     the range ``[height - content_height, 0]`` are automatically clipped in
     range.
@@ -1695,14 +1701,14 @@ class ScrollableTextLayout(TextLayout):
 class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
     '''Displayed text suitable for interactive editing and/or scrolling
     large documents.
-    
+
     Unlike `TextLayout` and `ScrollableTextLayout`, this class generates
     vertex lists only for lines of text that are visible.  As the document is
     scrolled, vertex lists are deleted and created as appropriate to keep
     video memory usage to a minimum and improve rendering speed.
 
     Changes to the document are quickly reflected in this layout, as only the
-    affected line(s) are reflowed.  Use `begin_update` and `end_update` to 
+    affected line(s) are reflowed.  Use `begin_update` and `end_update` to
     further reduce the amount of processing required.
 
     The layout can also display a text selection (text with a different
@@ -1752,6 +1758,10 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
     def delete(self):
         for line in self.lines:
             line.delete(self)
+        self.batch = None
+        if self._document:
+            self._document.remove_handlers(self)
+        self._document = None
 
     def on_insert_text(self, start, text):
         len_text = len(text)
@@ -1845,6 +1855,19 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
         if invalid_end - invalid_start <= 0:
             return
 
+        # Find grapheme breaks and extend glyph range to encompass.
+        text = self.document.text
+        while invalid_start > 0:
+            if _grapheme_break(text[invalid_start - 1], text[invalid_start]):
+                break
+            invalid_start -= 1
+
+        len_text = len(text)
+        while invalid_end < len_text:
+            if _grapheme_break(text[invalid_end - 1], text[invalid_end]):
+                break
+            invalid_end += 1
+
         # Update glyphs
         runs = runlist.ZipRunIterator((
             self._document.get_font_runs(dpi=self._dpi),
@@ -1869,7 +1892,7 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
 
         if invalid_end - invalid_start <= 0:
             return
-        
+
         # Find first invalid line
         line_index = 0
         for i, line in enumerate(self.lines):
@@ -1909,7 +1932,7 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
                 old_line.delete(self)
                 old_line_width = old_line.width + old_line.margin_left
                 new_line_width = line.width + line.margin_left
-                if (old_line_width == self.content_width and 
+                if (old_line_width == self.content_width and
                     new_line_width < old_line_width):
                     content_width_invalid = True
                 self.lines[line_index] = line
@@ -1917,10 +1940,10 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
             except IndexError:
                 self.lines.append(line)
                 self.invalid_lines.insert(line_index, 1)
-                
+
             next_start = line.start + line.length
             line_index += 1
-                
+
             try:
                 next_line = self.lines[line_index]
                 if next_start == next_line.start and next_start > invalid_end:
@@ -1971,7 +1994,7 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
             self.lines[i].delete(self)
         for i in range(end, min(self.visible_lines.end, len(self.lines))):
             self.lines[i].delete(self)
-        
+
         # Invalidate newly visible lines
         self.invalid_vertex_lines.invalidate(start, self.visible_lines.start)
         self.invalid_vertex_lines.invalidate(self.visible_lines.end, end)
@@ -1985,28 +2008,26 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
         self.invalid_vertex_lines.invalidate(
             self.get_line_from_position(style_invalid_start),
             self.get_line_from_position(style_invalid_end) + 1)
-        
+
         invalid_start, invalid_end = self.invalid_vertex_lines.validate()
         if invalid_end - invalid_start <= 0:
             return
 
-        batch = self.batch
-
         colors_iter = self.document.get_style_runs('color')
         background_iter = self.document.get_style_runs('background_color')
         if self._selection_end - self._selection_start > 0:
             colors_iter = runlist.OverriddenRunIterator(
                 colors_iter,
-                self._selection_start, 
+                self._selection_start,
                 self._selection_end,
                 self._selection_color)
             background_iter = runlist.OverriddenRunIterator(
                 background_iter,
-                self._selection_start, 
+                self._selection_start,
                 self._selection_end,
                 self._selection_background_color)
 
-        context = _IncrementalLayoutContext(self, self._document, 
+        context = _IncrementalLayoutContext(self, self._document,
                                             colors_iter, background_iter)
 
         for line in self.lines[invalid_start:invalid_end]:
@@ -2055,7 +2076,7 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
     def _set_multiline(self, multiline):
         self.invalid_flow.invalidate(0, len(self.document.text))
         super(IncrementalTextLayout, self)._set_multiline(multiline)
-        
+
     def _get_multiline(self):
         return self._multiline
 
@@ -2101,7 +2122,7 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
                                           max(end, self._selection_end))
         else:
             # Non-overlapping, invalidate both ranges
-            self.invalid_style.invalidate(self._selection_start, 
+            self.invalid_style.invalidate(self._selection_start,
                                           self._selection_end)
             self.invalid_style.invalidate(start, end)
 
@@ -2154,7 +2175,7 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
         self.invalid_style.invalidate(self._selection_start,
                                       self._selection_end)
 
-    selection_background_color = property(_get_selection_background_color, 
+    selection_background_color = property(_get_selection_background_color,
                                           _set_selection_background_color,
                                           doc='''Background color of active
     selection.
@@ -2178,7 +2199,7 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
         '''
         line = self.get_line_from_point(x, y)
         return self.get_position_on_line(line, x)
-    
+
     def get_point_from_position(self, position, line=None):
         '''Get the X, Y coordinates of a position in the document.
 
@@ -2205,14 +2226,14 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
                 line = next_line
         else:
             line = self.lines[line]
-            
+
         x = line.x
 
         baseline = self._document.get_style('baseline', max(0, position - 1))
         if baseline is None:
             baseline = 0
         else:
-            baseline = self._parse_distance(baseline) 
+            baseline = self._parse_distance(baseline)
 
         position -= line.start
         for box in line.boxes:
@@ -2221,8 +2242,8 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
                 break
             position -= box.length
             x += box.advance
-       
-        return (x + self.top_group.translate_x, 
+
+        return (x + self.top_group.translate_x,
                 line.y + self.top_group.translate_y + baseline)
 
     def get_line_from_point(self, x, y):
@@ -2259,7 +2280,7 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
         :return: (x, y)
         '''
         line = self.lines[line]
-        return (line.x + self.top_group.translate_x, 
+        return (line.x + self.top_group.translate_x,
                 line.y + self.top_group.translate_y)
 
     def get_line_from_position(self, position):
@@ -2351,13 +2372,13 @@ class IncrementalTextLayout(ScrollableTextLayout, event.EventDispatcher):
         if x <= self.view_x + 10:
             self.view_x = x - 10
         elif x >= self.view_x + self.width:
-            self.view_x = x - self.width + 10 
-        elif (x >= self.view_x + self.width - 10 and 
+            self.view_x = x - self.width + 10
+        elif (x >= self.view_x + self.width - 10 and
               self.content_width > self.width):
-            self.view_x = x - self.width + 10 
+            self.view_x = x - self.width + 10
 
     if _is_epydoc:
-        def on_layout_update():
+        def on_layout_update(self):
             '''Some or all of the layout text was reflowed.
 
             Text reflow is caused by document edits or changes to the layout's
