@@ -8,7 +8,7 @@ end the test.
 '''
 
 __docformat__ = 'restructuredtext'
-__version__ = '$Id: MATRIX_RGBA.py 1393 2007-11-10 01:09:58Z Alex.Holkner $'
+__version__ = '$Id: MATRIX_RGBA.py 1768 2008-02-17 12:45:54Z Alex.Holkner $'
 
 import unittest
 import base_load
@@ -26,8 +26,7 @@ class TEST_MATRIX_RGBA(base_load.TestLoad):
         else:
             # Load image as usual then rearrange components
             super(TEST_MATRIX_RGBA, self).load_image()
-            self.image.format = 'GBAR'
-            pixels = self.image.data # forces conversion
+            pixels = self.image.get_data('GBAR', self.image.width * 4)
 
 if __name__ == '__main__':
     unittest.main()
